@@ -1,7 +1,7 @@
 /*
     net_packet.c -- Handles in- and outgoing VPN packets
     Copyright (C) 1998-2005 Ivo Timmermans,
-                  2000-2006 Guus Sliepen <guus@tinc-vpn.org>
+                  2000-2009 Guus Sliepen <guus@tinc-vpn.org>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -13,11 +13,9 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
-
-    $Id: net_packet.c 1474 2006-11-29 17:18:39Z guus $
+    You should have received a copy of the GNU General Public License along
+    with this program; if not, write to the Free Software Foundation, Inc.,
+    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */
 
 #include "system.h"
@@ -48,75 +46,107 @@
 #include "utils.h"
 #include "xalloc.h"
 
-#ifdef WSAEMSGSIZE
-#define EMSGSIZE WSAEMSGSIZE
-#endif
-
 int keylifetime = 0;
 int keyexpires = 0;
-EVP_CIPHER_CTX packet_ctx;
 static char lzo_wrkmem[LZO1X_999_MEM_COMPRESS > LZO1X_1_MEM_COMPRESS ? LZO1X_999_MEM_COMPRESS : LZO1X_1_MEM_COMPRESS];
 
 static void send_udppacket(node_t *, vpn_packet_t *);
 
 #define MAX_SEQNO 1073741824
 
-void send_mtu_probe(node_t *n)
-{
+// mtuprobes == 1..30: initial discovery, send bursts with 1 second interval
+// mtuprobes ==    31: sleep pinginterval seconds
+// mtuprobes ==    32: send 1 burst, sleep pingtimeout second
+// mtuprobes ==    33: no response from other side, restart PMTU discovery process
+
+void send_mtu_probe(node_t *n) {
 	vpn_packet_t packet;
 	int len, i;
+	int timeout = 1;
 	
-	cp();
-
 	n->mtuprobes++;
 	n->mtuevent = NULL;
 
-	if(n->mtuprobes >= 10 && !n->minmtu) {
-		ifdebug(TRAFFIC) logger(LOG_INFO, _("No response to MTU probes from %s (%s)"), n->name, n->hostname);
+	if(!n->status.reachable || !n->status.validkey) {
+		ifdebug(TRAFFIC) logger(LOG_INFO, "Trying to send MTU probe to unreachable or rekeying node %s (%s)", n->name, n->hostname);
+		n->mtuprobes = 0;
 		return;
 	}
 
+	if(n->mtuprobes > 32) {
+		ifdebug(TRAFFIC) logger(LOG_INFO, "%s (%s) did not respond to UDP ping, restarting PMTU discovery", n->name, n->hostname);
+		n->mtuprobes = 1;
+		n->minmtu = 0;
+		n->maxmtu = MTU;
+	}
+
+	if(n->mtuprobes >= 10 && !n->minmtu) {
+		ifdebug(TRAFFIC) logger(LOG_INFO, "No response to MTU probes from %s (%s)", n->name, n->hostname);
+		n->mtuprobes = 0;
+		return;
+	}
+
+	if(n->mtuprobes == 30 || (n->mtuprobes < 30 && n->minmtu >= n->maxmtu)) {
+		if(n->minmtu > n->maxmtu)
+			n->minmtu = n->maxmtu;
+		else
+			n->maxmtu = n->minmtu;
+		n->mtu = n->minmtu;
+		ifdebug(TRAFFIC) logger(LOG_INFO, "Fixing MTU of %s (%s) to %d after %d probes", n->name, n->hostname, n->mtu, n->mtuprobes);
+		n->mtuprobes = 31;
+	}
+
+	if(n->mtuprobes == 31) {
+		timeout = pinginterval;
+		goto end;
+	} else if(n->mtuprobes == 32) {
+		timeout = pingtimeout;
+	}
+
 	for(i = 0; i < 3; i++) {
-		if(n->mtuprobes >= 30 || n->minmtu >= n->maxmtu) {
-			n->mtu = n->minmtu;
-			ifdebug(TRAFFIC) logger(LOG_INFO, _("Fixing MTU of %s (%s) to %d after %d probes"), n->name, n->hostname, n->mtu, n->mtuprobes);
-			return;
-		}
+		if(n->maxmtu <= n->minmtu)
+			len = n->maxmtu;
+		else
+			len = n->minmtu + 1 + rand() % (n->maxmtu - n->minmtu);
 
-		len = n->minmtu + 1 + random() % (n->maxmtu - n->minmtu);
 		if(len < 64)
 			len = 64;
 		
 		memset(packet.data, 0, 14);
 		RAND_pseudo_bytes(packet.data + 14, len - 14);
 		packet.len = len;
+		packet.priority = 0;
 
-		ifdebug(TRAFFIC) logger(LOG_INFO, _("Sending MTU probe length %d to %s (%s)"), len, n->name, n->hostname);
+		ifdebug(TRAFFIC) logger(LOG_INFO, "Sending MTU probe length %d to %s (%s)", len, n->name, n->hostname);
 
 		send_udppacket(n, &packet);
 	}
 
+end:
 	n->mtuevent = new_event();
 	n->mtuevent->handler = (event_handler_t)send_mtu_probe;
 	n->mtuevent->data = n;
-	n->mtuevent->time = now + 1;
+	n->mtuevent->time = now + timeout;
 	event_add(n->mtuevent);
 }
 
-void mtu_probe_h(node_t *n, vpn_packet_t *packet) {
-	ifdebug(TRAFFIC) logger(LOG_INFO, _("Got MTU probe length %d from %s (%s)"), packet->len, n->name, n->hostname);
+void mtu_probe_h(node_t *n, vpn_packet_t *packet, length_t len) {
+	ifdebug(TRAFFIC) logger(LOG_INFO, "Got MTU probe length %d from %s (%s)", packet->len, n->name, n->hostname);
 
 	if(!packet->data[0]) {
 		packet->data[0] = 1;
-		send_packet(n, packet);
+		send_udppacket(n, packet);
 	} else {
-		if(n->minmtu < packet->len)
-			n->minmtu = packet->len;
+		if(len > n->maxmtu)
+			len = n->maxmtu;
+		if(n->minmtu < len)
+			n->minmtu = len;
+		if(n->mtuprobes > 30)
+			n->mtuprobes = 30;
 	}
 }
 
-static length_t compress_packet(uint8_t *dest, const uint8_t *source, length_t len, int level)
-{
+static length_t compress_packet(uint8_t *dest, const uint8_t *source, length_t len, int level) {
 	if(level == 10) {
 		lzo_uint lzolen = MAXSIZE;
 		lzo1x_1_compress(source, len, dest, &lzolen, lzo_wrkmem);
@@ -136,8 +166,7 @@ static length_t compress_packet(uint8_t *dest, const uint8_t *source, length_t l
 	return -1;
 }
 
-static length_t uncompress_packet(uint8_t *dest, const uint8_t *source, length_t len, int level)
-{
+static length_t uncompress_packet(uint8_t *dest, const uint8_t *source, length_t len, int level) {
 	if(level > 9) {
 		lzo_uint lzolen = MAXSIZE;
 		if(lzo1x_decompress_safe(source, len, dest, &lzolen, NULL) == LZO_E_OK)
@@ -157,18 +186,25 @@ static length_t uncompress_packet(uint8_t *dest, const uint8_t *source, length_t
 
 /* VPN packet I/O */
 
-static void receive_packet(node_t *n, vpn_packet_t *packet)
-{
-	cp();
-
-	ifdebug(TRAFFIC) logger(LOG_DEBUG, _("Received packet of %d bytes from %s (%s)"),
+static void receive_packet(node_t *n, vpn_packet_t *packet) {
+	ifdebug(TRAFFIC) logger(LOG_DEBUG, "Received packet of %d bytes from %s (%s)",
 			   packet->len, n->name, n->hostname);
 
 	route(n, packet);
 }
 
-static void receive_udppacket(node_t *n, vpn_packet_t *inpkt)
-{
+static bool try_mac(const node_t *n, const vpn_packet_t *inpkt) {
+	unsigned char hmac[EVP_MAX_MD_SIZE];
+
+	if(!n->indigest || !n->inmaclength || !n->inkey || inpkt->len < sizeof inpkt->seqno + n->inmaclength)
+		return false;
+
+	HMAC(n->indigest, n->inkey, n->inkeylength, (unsigned char *) &inpkt->seqno, inpkt->len - n->inmaclength, (unsigned char *)hmac, NULL);
+
+	return !memcmp(hmac, (char *) &inpkt->seqno + inpkt->len - n->inmaclength, n->inmaclength);
+}
+
+static void receive_udppacket(node_t *n, vpn_packet_t *inpkt) {
 	vpn_packet_t pkt1, pkt2;
 	vpn_packet_t *pkt[] = { &pkt1, &pkt2, &pkt1, &pkt2 };
 	int nextpkt = 0;
@@ -177,25 +213,29 @@ static void receive_udppacket(node_t *n, vpn_packet_t *inpkt)
 	unsigned char hmac[EVP_MAX_MD_SIZE];
 	int i;
 
-	cp();
+	if(!n->inkey) {
+		ifdebug(TRAFFIC) logger(LOG_DEBUG, "Got packet from %s (%s) but he hasn't got our key yet",
+					n->name, n->hostname);
+		return;
+	}
 
 	/* Check packet length */
 
-	if(inpkt->len < sizeof(inpkt->seqno) + myself->maclength) {
-		ifdebug(TRAFFIC) logger(LOG_DEBUG, _("Got too short packet from %s (%s)"),
+	if(inpkt->len < sizeof(inpkt->seqno) + n->inmaclength) {
+		ifdebug(TRAFFIC) logger(LOG_DEBUG, "Got too short packet from %s (%s)",
 					n->name, n->hostname);
 		return;
 	}
 
 	/* Check the message authentication code */
 
-	if(myself->digest && myself->maclength) {
-		inpkt->len -= myself->maclength;
-		HMAC(myself->digest, myself->key, myself->keylength,
+	if(n->indigest && n->inmaclength) {
+		inpkt->len -= n->inmaclength;
+		HMAC(n->indigest, n->inkey, n->inkeylength,
 			 (unsigned char *) &inpkt->seqno, inpkt->len, (unsigned char *)hmac, NULL);
 
-		if(memcmp(hmac, (char *) &inpkt->seqno + inpkt->len, myself->maclength)) {
-			ifdebug(TRAFFIC) logger(LOG_DEBUG, _("Got unauthenticated packet from %s (%s)"),
+		if(memcmp(hmac, (char *) &inpkt->seqno + inpkt->len, n->inmaclength)) {
+			ifdebug(TRAFFIC) logger(LOG_DEBUG, "Got unauthenticated packet from %s (%s)",
 					   n->name, n->hostname);
 			return;
 		}
@@ -203,14 +243,14 @@ static void receive_udppacket(node_t *n, vpn_packet_t *inpkt)
 
 	/* Decrypt the packet */
 
-	if(myself->cipher) {
+	if(n->incipher) {
 		outpkt = pkt[nextpkt++];
 
-		if(!EVP_DecryptInit_ex(&packet_ctx, NULL, NULL, NULL, NULL)
-				|| !EVP_DecryptUpdate(&packet_ctx, (unsigned char *) &outpkt->seqno, &outlen,
+		if(!EVP_DecryptInit_ex(&n->inctx, NULL, NULL, NULL, NULL)
+				|| !EVP_DecryptUpdate(&n->inctx, (unsigned char *) &outpkt->seqno, &outlen,
 					(unsigned char *) &inpkt->seqno, inpkt->len)
-				|| !EVP_DecryptFinal_ex(&packet_ctx, (unsigned char *) &outpkt->seqno + outlen, &outpad)) {
-			ifdebug(TRAFFIC) logger(LOG_DEBUG, _("Error decrypting packet from %s (%s): %s"),
+				|| !EVP_DecryptFinal_ex(&n->inctx, (unsigned char *) &outpkt->seqno + outlen, &outpad)) {
+			ifdebug(TRAFFIC) logger(LOG_DEBUG, "Error decrypting packet from %s (%s): %s",
 						n->name, n->hostname, ERR_error_string(ERR_get_error(), NULL));
 			return;
 		}
@@ -226,13 +266,13 @@ static void receive_udppacket(node_t *n, vpn_packet_t *inpkt)
 
 	if(inpkt->seqno != n->received_seqno + 1) {
 		if(inpkt->seqno >= n->received_seqno + sizeof(n->late) * 8) {
-			logger(LOG_WARNING, _("Lost %d packets from %s (%s)"),
+			logger(LOG_WARNING, "Lost %d packets from %s (%s)",
 					   inpkt->seqno - n->received_seqno - 1, n->name, n->hostname);
 			
 			memset(n->late, 0, sizeof(n->late));
 		} else if (inpkt->seqno <= n->received_seqno) {
 			if((n->received_seqno >= sizeof(n->late) * 8 && inpkt->seqno <= n->received_seqno - sizeof(n->late) * 8) || !(n->late[(inpkt->seqno / 8) % sizeof(n->late)] & (1 << inpkt->seqno % 8))) {
-				logger(LOG_WARNING, _("Got late or replayed packet from %s (%s), seqno %d, last received %d"),
+				logger(LOG_WARNING, "Got late or replayed packet from %s (%s), seqno %d, last received %d",
 					   n->name, n->hostname, inpkt->seqno, n->received_seqno);
 				return;
 			}
@@ -252,41 +292,44 @@ static void receive_udppacket(node_t *n, vpn_packet_t *inpkt)
 
 	/* Decompress the packet */
 
-	if(myself->compression) {
+	length_t origlen = inpkt->len;
+
+	if(n->incompression) {
 		outpkt = pkt[nextpkt++];
 
-		if((outpkt->len = uncompress_packet(outpkt->data, inpkt->data, inpkt->len, myself->compression)) < 0) {
-			ifdebug(TRAFFIC) logger(LOG_ERR, _("Error while uncompressing packet from %s (%s)"),
+		if((outpkt->len = uncompress_packet(outpkt->data, inpkt->data, inpkt->len, n->incompression)) < 0) {
+			ifdebug(TRAFFIC) logger(LOG_ERR, "Error while uncompressing packet from %s (%s)",
 				  		 n->name, n->hostname);
 			return;
 		}
 
 		inpkt = outpkt;
+
+		origlen -= MTU/64 + 20;
 	}
 
-	if(n->connection)
-		n->connection->last_ping_time = now;
+	inpkt->priority = 0;
 
 	if(!inpkt->data[12] && !inpkt->data[13])
-		mtu_probe_h(n, inpkt);
+		mtu_probe_h(n, inpkt, origlen);
 	else
 		receive_packet(n, inpkt);
 }
 
-void receive_tcppacket(connection_t *c, char *buffer, int len)
-{
+void receive_tcppacket(connection_t *c, char *buffer, int len) {
 	vpn_packet_t outpkt;
 
-	cp();
-
 	outpkt.len = len;
+	if(c->options & OPTION_TCPONLY)
+		outpkt.priority = 0;
+	else
+		outpkt.priority = -1;
 	memcpy(outpkt.data, buffer, len);
 
 	receive_packet(c->node, &outpkt);
 }
 
-static void send_udppacket(node_t *n, vpn_packet_t *origpkt)
-{
+static void send_udppacket(node_t *n, vpn_packet_t *origpkt) {
 	vpn_packet_t pkt1, pkt2;
 	vpn_packet_t *pkt[] = { &pkt1, &pkt2, &pkt1, &pkt2 };
 	vpn_packet_t *inpkt = origpkt;
@@ -294,34 +337,42 @@ static void send_udppacket(node_t *n, vpn_packet_t *origpkt)
 	vpn_packet_t *outpkt;
 	int origlen;
 	int outlen, outpad;
-	vpn_packet_t *copy;
 	static int priority = 0;
 	int origpriority;
 	int sock;
 
-	cp();
+	if(!n->status.reachable) {
+		ifdebug(TRAFFIC) logger(LOG_INFO, "Trying to send UDP packet to unreachable node %s (%s)", n->name, n->hostname);
+		return;
+	}
 
 	/* Make sure we have a valid key */
 
 	if(!n->status.validkey) {
 		ifdebug(TRAFFIC) logger(LOG_INFO,
-				   _("No valid key known yet for %s (%s), queueing packet"),
+				   "No valid key known yet for %s (%s), forwarding via TCP",
 				   n->name, n->hostname);
 
-		/* Since packet is on the stack of handle_tap_input(), we have to make a copy of it first. */
-
-		*(copy = xmalloc(sizeof(*copy))) = *inpkt;
-
-		list_insert_tail(n->queue, copy);
-
-		if(n->queue->count > MAXQUEUELENGTH)
-			list_delete_head(n->queue);
-
 		if(!n->status.waitingforkey)
-			send_req_key(n->nexthop->connection, myself, n);
+			send_req_key(n);
 
 		n->status.waitingforkey = true;
 
+		send_tcppacket(n->nexthop->connection, origpkt);
+
+		return;
+	}
+
+	if(n->options & OPTION_PMTU_DISCOVERY && inpkt->len > n->minmtu && (inpkt->data[12] | inpkt->data[13])) {
+		ifdebug(TRAFFIC) logger(LOG_INFO,
+				"Packet for %s (%s) larger than minimum MTU, forwarding via %s",
+				n->name, n->hostname, n != n->nexthop ? n->nexthop->name : "TCP");
+
+		if(n != n->nexthop)
+			send_packet(n->nexthop, origpkt);
+		else
+			send_tcppacket(n->nexthop->connection, origpkt);
+
 		return;
 	}
 
@@ -330,11 +381,11 @@ static void send_udppacket(node_t *n, vpn_packet_t *origpkt)
 
 	/* Compress the packet */
 
-	if(n->compression) {
+	if(n->outcompression) {
 		outpkt = pkt[nextpkt++];
 
-		if((outpkt->len = compress_packet(outpkt->data, inpkt->data, inpkt->len, n->compression)) < 0) {
-			ifdebug(TRAFFIC) logger(LOG_ERR, _("Error while compressing packet to %s (%s)"),
+		if((outpkt->len = compress_packet(outpkt->data, inpkt->data, inpkt->len, n->outcompression)) < 0) {
+			ifdebug(TRAFFIC) logger(LOG_ERR, "Error while compressing packet to %s (%s)",
 				   n->name, n->hostname);
 			return;
 		}
@@ -349,14 +400,14 @@ static void send_udppacket(node_t *n, vpn_packet_t *origpkt)
 
 	/* Encrypt the packet */
 
-	if(n->cipher) {
+	if(n->outcipher) {
 		outpkt = pkt[nextpkt++];
 
-		if(!EVP_EncryptInit_ex(&n->packet_ctx, NULL, NULL, NULL, NULL)
-				|| !EVP_EncryptUpdate(&n->packet_ctx, (unsigned char *) &outpkt->seqno, &outlen,
+		if(!EVP_EncryptInit_ex(&n->outctx, NULL, NULL, NULL, NULL)
+				|| !EVP_EncryptUpdate(&n->outctx, (unsigned char *) &outpkt->seqno, &outlen,
 					(unsigned char *) &inpkt->seqno, inpkt->len)
-				|| !EVP_EncryptFinal_ex(&n->packet_ctx, (unsigned char *) &outpkt->seqno + outlen, &outpad)) {
-			ifdebug(TRAFFIC) logger(LOG_ERR, _("Error while encrypting packet to %s (%s): %s"),
+				|| !EVP_EncryptFinal_ex(&n->outctx, (unsigned char *) &outpkt->seqno + outlen, &outpad)) {
+			ifdebug(TRAFFIC) logger(LOG_ERR, "Error while encrypting packet to %s (%s): %s",
 						n->name, n->hostname, ERR_error_string(ERR_get_error(), NULL));
 			goto end;
 		}
@@ -367,10 +418,10 @@ static void send_udppacket(node_t *n, vpn_packet_t *origpkt)
 
 	/* Add the message authentication code */
 
-	if(n->digest && n->maclength) {
-		HMAC(n->digest, n->key, n->keylength, (unsigned char *) &inpkt->seqno,
+	if(n->outdigest && n->outmaclength) {
+		HMAC(n->outdigest, n->outkey, n->outkeylength, (unsigned char *) &inpkt->seqno,
 			 inpkt->len, (unsigned char *) &inpkt->seqno + inpkt->len, NULL);
-		inpkt->len += n->maclength;
+		inpkt->len += n->outmaclength;
 	}
 
 	/* Determine which socket we have to use */
@@ -388,20 +439,20 @@ static void send_udppacket(node_t *n, vpn_packet_t *origpkt)
 	if(priorityinheritance && origpriority != priority
 	   && listen_socket[sock].sa.sa.sa_family == AF_INET) {
 		priority = origpriority;
-		ifdebug(TRAFFIC) logger(LOG_DEBUG, _("Setting outgoing packet priority to %d"), priority);
+		ifdebug(TRAFFIC) logger(LOG_DEBUG, "Setting outgoing packet priority to %d", priority);
 		if(setsockopt(listen_socket[sock].udp, SOL_IP, IP_TOS, &priority, sizeof(priority)))	/* SO_PRIORITY doesn't seem to work */
-			logger(LOG_ERR, _("System call `%s' failed: %s"), "setsockopt", strerror(errno));
+			logger(LOG_ERR, "System call `%s' failed: %s", "setsockopt", strerror(errno));
 	}
 #endif
 
-	if((sendto(listen_socket[sock].udp, (char *) &inpkt->seqno, inpkt->len, 0, &(n->address.sa), SALEN(n->address.sa))) < 0) {
-		if(errno == EMSGSIZE) {
+	if(sendto(listen_socket[sock].udp, (char *) &inpkt->seqno, inpkt->len, 0, &(n->address.sa), SALEN(n->address.sa)) < 0 && !sockwouldblock(sockerrno)) {
+		if(sockmsgsize(sockerrno)) {
 			if(n->maxmtu >= origlen)
 				n->maxmtu = origlen - 1;
 			if(n->mtu >= origlen)
 				n->mtu = origlen - 1;
 		} else
-			logger(LOG_ERR, _("Error sending packet to %s (%s): %s"), n->name, n->hostname, strerror(errno));
+			logger(LOG_ERR, "Error sending packet to %s (%s): %s", n->name, n->hostname, sockstrerror(sockerrno));
 	}
 
 end:
@@ -411,12 +462,9 @@ end:
 /*
   send a packet to the given vpn ip.
 */
-void send_packet(const node_t *n, vpn_packet_t *packet)
-{
+void send_packet(const node_t *n, vpn_packet_t *packet) {
 	node_t *via;
 
-	cp();
-
 	if(n == myself) {
 		if(overwrite_mac)
 			 memcpy(packet->data, mymac.x, ETH_ALEN);
@@ -424,22 +472,22 @@ void send_packet(const node_t *n, vpn_packet_t *packet)
 		return;
 	}
 
-	ifdebug(TRAFFIC) logger(LOG_ERR, _("Sending packet of %d bytes to %s (%s)"),
+	ifdebug(TRAFFIC) logger(LOG_ERR, "Sending packet of %d bytes to %s (%s)",
 			   packet->len, n->name, n->hostname);
 
 	if(!n->status.reachable) {
-		ifdebug(TRAFFIC) logger(LOG_INFO, _("Node %s (%s) is not reachable"),
+		ifdebug(TRAFFIC) logger(LOG_INFO, "Node %s (%s) is not reachable",
 				   n->name, n->hostname);
 		return;
 	}
 
-	via = (n->via == myself) ? n->nexthop : n->via;
+	via = (packet->priority == -1 || n->via == myself) ? n->nexthop : n->via;
 
 	if(via != n)
-		ifdebug(TRAFFIC) logger(LOG_ERR, _("Sending packet to %s via %s (%s)"),
+		ifdebug(TRAFFIC) logger(LOG_INFO, "Sending packet to %s via %s (%s)",
 			   n->name, via->name, n->via->hostname);
 
-	if((myself->options | via->options) & OPTION_TCPONLY) {
+	if(packet->priority == -1 || ((myself->options | via->options) & OPTION_TCPONLY)) {
 		if(!send_tcppacket(via->connection, packet))
 			terminate_connection(via->connection, true);
 	} else
@@ -448,19 +496,22 @@ void send_packet(const node_t *n, vpn_packet_t *packet)
 
 /* Broadcast a packet using the minimum spanning tree */
 
-void broadcast_packet(const node_t *from, vpn_packet_t *packet)
-{
+void broadcast_packet(const node_t *from, vpn_packet_t *packet) {
 	avl_node_t *node;
 	connection_t *c;
 
-	cp();
-
-	ifdebug(TRAFFIC) logger(LOG_INFO, _("Broadcasting packet of %d bytes from %s (%s)"),
+	ifdebug(TRAFFIC) logger(LOG_INFO, "Broadcasting packet of %d bytes from %s (%s)",
 			   packet->len, from->name, from->hostname);
 
-	if(from != myself)
+	if(from != myself) {
 		send_packet(myself, packet);
 
+		// In TunnelServer mode, do not forward broadcast packets.
+                // The MST might not be valid and create loops.
+		if(tunnelserver)
+			return;
+	}
+
 	for(node = connection_tree->head; node; node = node->next) {
 		c = node->data;
 
@@ -469,35 +520,46 @@ void broadcast_packet(const node_t *from, vpn_packet_t *packet)
 	}
 }
 
-void flush_queue(node_t *n)
-{
-	list_node_t *node, *next;
+static node_t *try_harder(const sockaddr_t *from, const vpn_packet_t *pkt) {
+	avl_node_t *node;
+	edge_t *e;
+	node_t *n = NULL;
+	static time_t last_hard_try = 0;
 
-	cp();
+	for(node = edge_weight_tree->head; node; node = node->next) {
+		e = node->data;
 
-	ifdebug(TRAFFIC) logger(LOG_INFO, _("Flushing queue for %s (%s)"), n->name, n->hostname);
+		if(sockaddrcmp_noport(from, &e->address)) {
+			if(last_hard_try == now)
+				continue;
+			last_hard_try = now;
+		}
 
-	for(node = n->queue->head; node; node = next) {
-		next = node->next;
-		send_udppacket(n, node->data);
-		list_delete_node(n->queue, node);
+		if(!n)
+			n = e->to;
+
+		if(!try_mac(e->to, pkt))
+			continue;
+
+		n = e->to;
+		break;
 	}
+
+	return n;
 }
 
-void handle_incoming_vpn_data(int sock)
-{
+void handle_incoming_vpn_data(int sock) {
 	vpn_packet_t pkt;
 	char *hostname;
 	sockaddr_t from;
 	socklen_t fromlen = sizeof(from);
 	node_t *n;
 
-	cp();
-
 	pkt.len = recvfrom(sock, (char *) &pkt.seqno, MAXSIZE, 0, &from.sa, &fromlen);
 
 	if(pkt.len < 0) {
-		logger(LOG_ERR, _("Receiving packet failed: %s"), strerror(errno));
+		if(!sockwouldblock(sockerrno))
+			logger(LOG_ERR, "Receiving packet failed: %s", sockstrerror(sockerrno));
 		return;
 	}
 
@@ -506,11 +568,17 @@ void handle_incoming_vpn_data(int sock)
 	n = lookup_node_udp(&from);
 
 	if(!n) {
-		hostname = sockaddr2hostname(&from);
-		logger(LOG_WARNING, _("Received UDP packet from unknown source %s"),
-			   hostname);
-		free(hostname);
-		return;
+		n = try_harder(&from, &pkt);
+		if(n)
+			update_node_udp(n, &from);
+		else ifdebug(PROTOCOL) {
+			hostname = sockaddr2hostname(&from);
+			logger(LOG_WARNING, "Received UDP packet from unknown source %s", hostname);
+			free(hostname);
+			return;
+		}
+		else
+			return;
 	}
 
 	receive_udppacket(n, &pkt);
