@@ -1,7 +1,7 @@
 /*
     protocol_key.c -- handle the meta-protocol, key exchange
     Copyright (C) 1999-2005 Ivo Timmermans,
-                  2000-2008 Guus Sliepen <guus@tinc-vpn.org>
+                  2000-2009 Guus Sliepen <guus@tinc-vpn.org>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -13,17 +13,16 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
-
-    $Id: protocol_key.c 1595 2008-12-22 20:27:52Z guus $
+    You should have received a copy of the GNU General Public License along
+    with this program; if not, write to the Free Software Foundation, Inc.,
+    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */
 
 #include "system.h"
 
 #include <openssl/evp.h>
 #include <openssl/err.h>
+#include <openssl/rand.h>
 
 #include "avl_tree.h"
 #include "connection.h"
@@ -37,29 +36,23 @@
 
 bool mykeyused = false;
 
-bool send_key_changed(connection_t *c, const node_t *n)
-{
-	cp();
-
+bool send_key_changed() {
 	/* Only send this message if some other daemon requested our key previously.
 	   This reduces unnecessary key_changed broadcasts.
 	 */
 
-	if(n == myself && !mykeyused)
+	if(!mykeyused)
 		return true;
 
-	return send_request(c, "%d %lx %s", KEY_CHANGED, random(), n->name);
+	return send_request(broadcast, "%d %x %s", KEY_CHANGED, rand(), myself->name);
 }
 
-bool key_changed_h(connection_t *c)
-{
+bool key_changed_h(connection_t *c) {
 	char name[MAX_STRING_SIZE];
 	node_t *n;
 
-	cp();
-
 	if(sscanf(c->buffer, "%*d %*x " MAX_STRING, name) != 1) {
-		logger(LOG_ERR, _("Got bad %s from %s (%s)"), "KEY_CHANGED",
+		logger(LOG_ERR, "Got bad %s from %s (%s)", "KEY_CHANGED",
 			   c->name, c->hostname);
 		return false;
 	}
@@ -70,7 +63,7 @@ bool key_changed_h(connection_t *c)
 	n = lookup_node(name);
 
 	if(!n) {
-		logger(LOG_ERR, _("Got %s from %s (%s) origin %s which does not exist"),
+		logger(LOG_ERR, "Got %s from %s (%s) origin %s which does not exist",
 			   "KEY_CHANGED", c->name, c->hostname, name);
 		return false;
 	}
@@ -86,23 +79,17 @@ bool key_changed_h(connection_t *c)
 	return true;
 }
 
-bool send_req_key(connection_t *c, const node_t *from, const node_t *to)
-{
-	cp();
-
-	return send_request(c, "%d %s %s", REQ_KEY, from->name, to->name);
+bool send_req_key(node_t *to) {
+	return send_request(to->nexthop->connection, "%d %s %s", REQ_KEY, myself->name, to->name);
 }
 
-bool req_key_h(connection_t *c)
-{
+bool req_key_h(connection_t *c) {
 	char from_name[MAX_STRING_SIZE];
 	char to_name[MAX_STRING_SIZE];
 	node_t *from, *to;
 
-	cp();
-
 	if(sscanf(c->buffer, "%*d " MAX_STRING " " MAX_STRING, from_name, to_name) != 2) {
-		logger(LOG_ERR, _("Got bad %s from %s (%s)"), "REQ_KEY", c->name,
+		logger(LOG_ERR, "Got bad %s from %s (%s)", "REQ_KEY", c->name,
 			   c->hostname);
 		return false;
 	}
@@ -110,7 +97,7 @@ bool req_key_h(connection_t *c)
 	from = lookup_node(from_name);
 
 	if(!from) {
-		logger(LOG_ERR, _("Got %s from %s (%s) origin %s which does not exist in our connection list"),
+		logger(LOG_ERR, "Got %s from %s (%s) origin %s which does not exist in our connection list",
 			   "REQ_KEY", c->name, c->hostname, from_name);
 		return false;
 	}
@@ -118,7 +105,7 @@ bool req_key_h(connection_t *c)
 	to = lookup_node(to_name);
 
 	if(!to) {
-		logger(LOG_ERR, _("Got %s from %s (%s) destination %s which does not exist in our connection list"),
+		logger(LOG_ERR, "Got %s from %s (%s) destination %s which does not exist in our connection list",
 			   "REQ_KEY", c->name, c->hostname, to_name);
 		return false;
 	}
@@ -126,57 +113,69 @@ bool req_key_h(connection_t *c)
 	/* Check if this key request is for us */
 
 	if(to == myself) {			/* Yes, send our own key back */
-		mykeyused = true;
-		from->received_seqno = 0;
-		memset(from->late, 0, sizeof(from->late));
-		send_ans_key(c, myself, from);
+		send_ans_key(from);
 	} else {
 		if(tunnelserver)
 			return false;
 
 		if(!to->status.reachable) {
-			logger(LOG_WARNING, _("Got %s from %s (%s) destination %s which is not reachable"),
+			logger(LOG_WARNING, "Got %s from %s (%s) destination %s which is not reachable",
 				"REQ_KEY", c->name, c->hostname, to_name);
 			return true;
 		}
 
-		send_req_key(to->nexthop->connection, from, to);
+		send_request(to->nexthop->connection, "%s", c->buffer);
 	}
 
 	return true;
 }
 
-bool send_ans_key(connection_t *c, const node_t *from, const node_t *to)
-{
+bool send_ans_key(node_t *to) {
 	char *key;
 
-	cp();
+	// Set key parameters
+	to->incipher = myself->incipher;
+	to->inkeylength = myself->inkeylength;
+	to->indigest = myself->indigest;
+	to->inmaclength = myself->inmaclength;
+	to->incompression = myself->incompression;
 
-	key = alloca(2 * from->keylength + 1);
-	bin2hex(from->key, key, from->keylength);
-	key[from->keylength * 2] = '\0';
+	// Allocate memory for key
+	to->inkey = xrealloc(to->inkey, to->inkeylength);
 
-	return send_request(c, "%d %s %s %s %d %d %d %d", ANS_KEY,
-						from->name, to->name, key,
-						from->cipher ? from->cipher->nid : 0,
-						from->digest ? from->digest->type : 0, from->maclength,
-						from->compression);
+	// Create a new key
+	RAND_pseudo_bytes((unsigned char *)to->inkey, to->inkeylength);
+	if(to->incipher)
+		EVP_DecryptInit_ex(&to->inctx, to->incipher, NULL, (unsigned char *)to->inkey, (unsigned char *)to->inkey + to->incipher->key_len);
+
+	// Reset sequence number and late packet window
+	mykeyused = true;
+	to->received_seqno = 0;
+	memset(to->late, 0, sizeof(to->late));
+
+	// Convert to hexadecimal and send
+	key = alloca(2 * to->inkeylength + 1);
+	bin2hex(to->inkey, key, to->inkeylength);
+	key[to->inkeylength * 2] = '\0';
+
+	return send_request(to->nexthop->connection, "%d %s %s %s %d %d %d %d", ANS_KEY,
+			myself->name, to->name, key,
+			to->incipher ? to->incipher->nid : 0,
+			to->indigest ? to->indigest->type : 0, to->inmaclength,
+			to->incompression);
 }
 
-bool ans_key_h(connection_t *c)
-{
+bool ans_key_h(connection_t *c) {
 	char from_name[MAX_STRING_SIZE];
 	char to_name[MAX_STRING_SIZE];
 	char key[MAX_STRING_SIZE];
 	int cipher, digest, maclength, compression;
 	node_t *from, *to;
 
-	cp();
-
 	if(sscanf(c->buffer, "%*d "MAX_STRING" "MAX_STRING" "MAX_STRING" %d %d %d %d",
 		from_name, to_name, key, &cipher, &digest, &maclength,
 		&compression) != 7) {
-		logger(LOG_ERR, _("Got bad %s from %s (%s)"), "ANS_KEY", c->name,
+		logger(LOG_ERR, "Got bad %s from %s (%s)", "ANS_KEY", c->name,
 			   c->hostname);
 		return false;
 	}
@@ -184,7 +183,7 @@ bool ans_key_h(connection_t *c)
 	from = lookup_node(from_name);
 
 	if(!from) {
-		logger(LOG_ERR, _("Got %s from %s (%s) origin %s which does not exist in our connection list"),
+		logger(LOG_ERR, "Got %s from %s (%s) origin %s which does not exist in our connection list",
 			   "ANS_KEY", c->name, c->hostname, from_name);
 		return false;
 	}
@@ -192,7 +191,7 @@ bool ans_key_h(connection_t *c)
 	to = lookup_node(to_name);
 
 	if(!to) {
-		logger(LOG_ERR, _("Got %s from %s (%s) destination %s which does not exist in our connection list"),
+		logger(LOG_ERR, "Got %s from %s (%s) destination %s which does not exist in our connection list",
 			   "ANS_KEY", c->name, c->hostname, to_name);
 		return false;
 	}
@@ -204,7 +203,7 @@ bool ans_key_h(connection_t *c)
 			return false;
 
 		if(!to->status.reachable) {
-			logger(LOG_WARNING, _("Got %s from %s (%s) destination %s which is not reachable"),
+			logger(LOG_WARNING, "Got %s from %s (%s) destination %s which is not reachable",
 				"ANS_KEY", c->name, c->hostname, to_name);
 			return true;
 		}
@@ -213,77 +212,72 @@ bool ans_key_h(connection_t *c)
 	}
 
 	/* Update our copy of the origin's packet key */
+	from->outkey = xrealloc(from->outkey, strlen(key) / 2);
 
-	if(from->key)
-		free(from->key);
+	from->outkey = xstrdup(key);
+	from->outkeylength = strlen(key) / 2;
+	hex2bin(key, from->outkey, from->outkeylength);
 
-	from->key = xstrdup(key);
-	from->keylength = strlen(key) / 2;
-	hex2bin(from->key, from->key, from->keylength);
-	from->key[from->keylength] = '\0';
-
-	from->status.validkey = true;
 	from->status.waitingforkey = false;
-	from->sent_seqno = 0;
-
 	/* Check and lookup cipher and digest algorithms */
 
 	if(cipher) {
-		from->cipher = EVP_get_cipherbynid(cipher);
+		from->outcipher = EVP_get_cipherbynid(cipher);
 
-		if(!from->cipher) {
-			logger(LOG_ERR, _("Node %s (%s) uses unknown cipher!"), from->name,
+		if(!from->outcipher) {
+			logger(LOG_ERR, "Node %s (%s) uses unknown cipher!", from->name,
 				   from->hostname);
 			return false;
 		}
 
-		if(from->keylength != from->cipher->key_len + from->cipher->iv_len) {
-			logger(LOG_ERR, _("Node %s (%s) uses wrong keylength!"), from->name,
+		if(from->outkeylength != from->outcipher->key_len + from->outcipher->iv_len) {
+			logger(LOG_ERR, "Node %s (%s) uses wrong keylength!", from->name,
 				   from->hostname);
 			return false;
 		}
 	} else {
-		from->cipher = NULL;
+		from->outcipher = NULL;
 	}
 
-	from->maclength = maclength;
+	from->outmaclength = maclength;
 
 	if(digest) {
-		from->digest = EVP_get_digestbynid(digest);
+		from->outdigest = EVP_get_digestbynid(digest);
 
-		if(!from->digest) {
-			logger(LOG_ERR, _("Node %s (%s) uses unknown digest!"), from->name,
+		if(!from->outdigest) {
+			logger(LOG_ERR, "Node %s (%s) uses unknown digest!", from->name,
 				   from->hostname);
 			return false;
 		}
 
-		if(from->maclength > from->digest->md_size || from->maclength < 0) {
-			logger(LOG_ERR, _("Node %s (%s) uses bogus MAC length!"),
+		if(from->outmaclength > from->outdigest->md_size || from->outmaclength < 0) {
+			logger(LOG_ERR, "Node %s (%s) uses bogus MAC length!",
 				   from->name, from->hostname);
 			return false;
 		}
 	} else {
-		from->digest = NULL;
+		from->outdigest = NULL;
 	}
 
 	if(compression < 0 || compression > 11) {
-		logger(LOG_ERR, _("Node %s (%s) uses bogus compression level!"), from->name, from->hostname);
+		logger(LOG_ERR, "Node %s (%s) uses bogus compression level!", from->name, from->hostname);
 		return false;
 	}
 	
-	from->compression = compression;
+	from->outcompression = compression;
 
-	if(from->cipher)
-		if(!EVP_EncryptInit_ex(&from->packet_ctx, from->cipher, NULL, (unsigned char *)from->key, (unsigned char *)from->key + from->cipher->key_len)) {
-			logger(LOG_ERR, _("Error during initialisation of key from %s (%s): %s"),
+	if(from->outcipher)
+		if(!EVP_EncryptInit_ex(&from->outctx, from->outcipher, NULL, (unsigned char *)from->outkey, (unsigned char *)from->outkey + from->outcipher->key_len)) {
+			logger(LOG_ERR, "Error during initialisation of key from %s (%s): %s",
 					from->name, from->hostname, ERR_error_string(ERR_get_error(), NULL));
 			return false;
 		}
 
+	from->status.validkey = true;
+	from->sent_seqno = 0;
+
 	if(from->options & OPTION_PMTU_DISCOVERY && !from->mtuprobes)
 		send_mtu_probe(from);
 
-	flush_queue(from);
-
 	return true;
 }
