@@ -1,7 +1,8 @@
 /*
     protocol_subnet.c -- handle the meta-protocol, subnets
     Copyright (C) 1999-2005 Ivo Timmermans,
-                  2000-2006 Guus Sliepen <guus@tinc-vpn.org>
+                  2000-2009 Guus Sliepen <guus@tinc-vpn.org>
+                  2009      Michael Tokarev <mjt@tls.msk.ru>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -13,11 +14,9 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
-
-    $Id: protocol_subnet.c 1452 2006-04-26 13:52:58Z guus $
+    You should have received a copy of the GNU General Public License along
+    with this program; if not, write to the Free Software Foundation, Inc.,
+    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */
 
 #include "system.h"
@@ -33,46 +32,40 @@
 #include "utils.h"
 #include "xalloc.h"
 
-bool send_add_subnet(connection_t *c, const subnet_t *subnet)
-{
+bool send_add_subnet(connection_t *c, const subnet_t *subnet) {
 	char netstr[MAXNETSTR];
 
-	cp();
-
 	if(!net2str(netstr, sizeof netstr, subnet))
 		return false;
 
-	return send_request(c, "%d %lx %s %s", ADD_SUBNET, random(), subnet->owner->name, netstr);
+	return send_request(c, "%d %x %s %s", ADD_SUBNET, rand(), subnet->owner->name, netstr);
 }
 
-bool add_subnet_h(connection_t *c)
-{
+bool add_subnet_h(connection_t *c) {
 	char subnetstr[MAX_STRING_SIZE];
 	char name[MAX_STRING_SIZE];
 	node_t *owner;
-	subnet_t s = {0}, *new;
-
-	cp();
+	subnet_t s = {0}, *new, *old;
 
 	if(sscanf(c->buffer, "%*d %*x " MAX_STRING " " MAX_STRING, name, subnetstr) != 2) {
-		logger(LOG_ERR, _("Got bad %s from %s (%s)"), "ADD_SUBNET", c->name,
+		logger(LOG_ERR, "Got bad %s from %s (%s)", "ADD_SUBNET", c->name,
 			   c->hostname);
 		return false;
 	}
 
-	/* Check if owner name is a valid */
+	/* Check if owner name is valid */
 
 	if(!check_id(name)) {
-		logger(LOG_ERR, _("Got bad %s from %s (%s): %s"), "ADD_SUBNET", c->name,
-			   c->hostname, _("invalid name"));
+		logger(LOG_ERR, "Got bad %s from %s (%s): %s", "ADD_SUBNET", c->name,
+			   c->hostname, "invalid name");
 		return false;
 	}
 
 	/* Check if subnet string is valid */
 
 	if(!str2net(&s, subnetstr)) {
-		logger(LOG_ERR, _("Got bad %s from %s (%s): %s"), "ADD_SUBNET", c->name,
-			   c->hostname, _("invalid subnet string"));
+		logger(LOG_ERR, "Got bad %s from %s (%s): %s", "ADD_SUBNET", c->name,
+			   c->hostname, "invalid subnet string");
 		return false;
 	}
 
@@ -83,15 +76,19 @@ bool add_subnet_h(connection_t *c)
 
 	owner = lookup_node(name);
 
+	if(tunnelserver && owner != myself && owner != c->node) {
+		/* in case of tunnelserver, ignore indirect subnet registrations */
+		ifdebug(PROTOCOL) logger(LOG_WARNING, "Ignoring indirect %s from %s (%s) for %s",
+				   "ADD_SUBNET", c->name, c->hostname, subnetstr);
+		return true;
+	}
+
 	if(!owner) {
 		owner = new_node();
 		owner->name = xstrdup(name);
 		node_add(owner);
 	}
 
-	if(tunnelserver && owner != myself && owner != c->node)
-		return false;
-
 	/* Check if we already know this subnet */
 
 	if(lookup_subnet(owner, &s))
@@ -100,7 +97,7 @@ bool add_subnet_h(connection_t *c)
 	/* If we don't know this subnet, but we are the owner, retaliate with a DEL_SUBNET */
 
 	if(owner == myself) {
-		ifdebug(PROTOCOL) logger(LOG_WARNING, _("Got %s from %s (%s) for ourself"),
+		ifdebug(PROTOCOL) logger(LOG_WARNING, "Got %s from %s (%s) for ourself",
 				   "ADD_SUBNET", c->name, c->hostname);
 		s.owner = myself;
 		send_del_subnet(c, &s);
@@ -115,7 +112,7 @@ bool add_subnet_h(connection_t *c)
 
 		for(cfg = lookup_config(c->config_tree, "Subnet"); cfg; cfg = lookup_config_next(c->config_tree, cfg)) {
 			if(!get_config_subnet(cfg, &allowed))
-				return false;
+				continue;
 
 			if(!subnet_compare(&s, allowed))
 				break;
@@ -123,8 +120,11 @@ bool add_subnet_h(connection_t *c)
 			free_subnet(allowed);
 		}
 
-		if(!cfg)
-			return false;
+		if(!cfg) {
+			logger(LOG_WARNING, "Ignoring unauthorized %s from %s (%s): %s",
+					"ADD_SUBNET", c->name, c->hostname, subnetstr);
+			return true;
+		}
 
 		free_subnet(allowed);
 	}
@@ -142,68 +142,71 @@ bool add_subnet_h(connection_t *c)
 	if(!tunnelserver)
 		forward_request(c);
 
+	/* Fast handoff of roaming MAC addresses */
+
+	if(s.type == SUBNET_MAC && owner != myself && (old = lookup_subnet(myself, &s)) && old->expires)
+		old->expires = now;
+
 	return true;
 }
 
-bool send_del_subnet(connection_t *c, const subnet_t *s)
-{
+bool send_del_subnet(connection_t *c, const subnet_t *s) {
 	char netstr[MAXNETSTR];
 
-	cp();
-
 	if(!net2str(netstr, sizeof netstr, s))
 		return false;
 
-	return send_request(c, "%d %lx %s %s", DEL_SUBNET, random(), s->owner->name, netstr);
+	return send_request(c, "%d %x %s %s", DEL_SUBNET, rand(), s->owner->name, netstr);
 }
 
-bool del_subnet_h(connection_t *c)
-{
+bool del_subnet_h(connection_t *c) {
 	char subnetstr[MAX_STRING_SIZE];
 	char name[MAX_STRING_SIZE];
 	node_t *owner;
 	subnet_t s = {0}, *find;
 
-	cp();
-
 	if(sscanf(c->buffer, "%*d %*x " MAX_STRING " " MAX_STRING, name, subnetstr) != 2) {
-		logger(LOG_ERR, _("Got bad %s from %s (%s)"), "DEL_SUBNET", c->name,
+		logger(LOG_ERR, "Got bad %s from %s (%s)", "DEL_SUBNET", c->name,
 			   c->hostname);
 		return false;
 	}
 
-	/* Check if owner name is a valid */
+	/* Check if owner name is valid */
 
 	if(!check_id(name)) {
-		logger(LOG_ERR, _("Got bad %s from %s (%s): %s"), "DEL_SUBNET", c->name,
-			   c->hostname, _("invalid name"));
+		logger(LOG_ERR, "Got bad %s from %s (%s): %s", "DEL_SUBNET", c->name,
+			   c->hostname, "invalid name");
 		return false;
 	}
 
-	/* Check if the owner of the new subnet is in the connection list */
-
-	owner = lookup_node(name);
-
-	if(!owner) {
-		ifdebug(PROTOCOL) logger(LOG_WARNING, _("Got %s from %s (%s) for %s which is not in our node tree"),
-				   "DEL_SUBNET", c->name, c->hostname, name);
-		return true;
-	}
-
-	if(tunnelserver && owner != myself && owner != c->node)
-		return false;
-
 	/* Check if subnet string is valid */
 
 	if(!str2net(&s, subnetstr)) {
-		logger(LOG_ERR, _("Got bad %s from %s (%s): %s"), "DEL_SUBNET", c->name,
-			   c->hostname, _("invalid subnet string"));
+		logger(LOG_ERR, "Got bad %s from %s (%s): %s", "DEL_SUBNET", c->name,
+			   c->hostname, "invalid subnet string");
 		return false;
 	}
 
 	if(seen_request(c->buffer))
 		return true;
 
+	/* Check if the owner of the subnet being deleted is in the connection list */
+
+	owner = lookup_node(name);
+
+	if(tunnelserver && owner != myself && owner != c->node) {
+		/* in case of tunnelserver, ignore indirect subnet deletion */
+		ifdebug(PROTOCOL) logger(LOG_WARNING, "Ignoring indirect %s from %s (%s) for %s",
+                                  "DEL_SUBNET", c->name, c->hostname, subnetstr);
+		return true;
+	}
+
+	if(!owner) {
+		ifdebug(PROTOCOL) logger(LOG_WARNING, "Got %s from %s (%s) for %s which is not in our node tree",
+				   "DEL_SUBNET", c->name, c->hostname, name);
+		return true;
+	}
+
 	/* If everything is correct, delete the subnet from the list of the owner */
 
 	s.owner = owner;
@@ -211,7 +214,7 @@ bool del_subnet_h(connection_t *c)
 	find = lookup_subnet(owner, &s);
 
 	if(!find) {
-		ifdebug(PROTOCOL) logger(LOG_WARNING, _("Got %s from %s (%s) for %s which does not appear in his subnet tree"),
+		ifdebug(PROTOCOL) logger(LOG_WARNING, "Got %s from %s (%s) for %s which does not appear in his subnet tree",
 				   "DEL_SUBNET", c->name, c->hostname, name);
 		return true;
 	}
@@ -219,7 +222,7 @@ bool del_subnet_h(connection_t *c)
 	/* If we are the owner of this subnet, retaliate with an ADD_SUBNET */
 
 	if(owner == myself) {
-		ifdebug(PROTOCOL) logger(LOG_WARNING, _("Got %s from %s (%s) for ourself"),
+		ifdebug(PROTOCOL) logger(LOG_WARNING, "Got %s from %s (%s) for ourself",
 				   "DEL_SUBNET", c->name, c->hostname);
 		send_add_subnet(c, find);
 		return true;
